from PySide2.QtGui import QTransform
from axipy import Pnt, provider_manager, CoordSystem, Feature, Schema


class Affine:
    def __init__(self,points_base,points_tab):
        self.__points_base=points_base
        self.__points_tab=points_tab
        self.__isValid=False
        self.__calc()
    def __calc(self):
        self._tx=self.__calcCoefficient(self.__points_tab[0],self.__points_tab[1],self.__points_tab[2],self.__points_base[0].x,self.__points_base[1].x,self.__points_base[2].x)
        self._ty = self.__calcCoefficient(self.__points_tab[0], self.__points_tab[1], self.__points_tab[2],
                                    self.__points_base[0].y, self.__points_base[1].y, self.__points_base[2].y)

    def __calcCoefficient(self,pnt1s:Pnt,pnt2s:Pnt,pnt3s:Pnt,v1o,v2o,v3o):
        d1_2x=pnt1s.x-pnt2s.x
        d1_2y=pnt1s.y-pnt2s.y
        d1_3x=pnt1s.x-pnt3s.x
        d1_3y = pnt1s.y - pnt3s.y
        t1_2=v1o-v2o
        t1_3=v1o-v3o
        if d1_3y==0:
            if d1_3x==0:
                self.__isValid=False
                return
            a=t1_3/d1_3x
            if d1_2y==0:
                self.__isValid = False
                return
        else:
            a6=d1_2x-d1_3x*d1_2y/d1_3y
            t6=t1_2-t1_3*d1_2y/d1_3y
            if a6==0:
                self.__isValid=False
                return
            a=t6/a6
        if d1_2y==0:
            b=(t1_3-d1_3x*a)/d1_3y
        else:
            b=(t1_2-d1_2x*a)/d1_2y
        c=v1o-a*pnt1s.x-b*pnt1s.y
        self.__isValid = True

        return a,b,c
    def getTransform(self):
        #qt_trans=QTransform(self._tx[0],self._ty[0],0,self._tx[1],self._ty[1],0,self._tx[2],self._ty[2],0)
        #qt_trans.setMatrix(self._tx[0],self._ty[0],1,self._tx[1],self._ty[1],1,self._tx[2],self._ty[2],1)
        qt_trans=QTransform(self._tx[0],self._ty[0], self._tx[1],self._ty[1], self._tx[2],self._ty[2])
        return qt_trans


class FeatureTransform:
    def __init__(self,transform,out_cs):
        self.__transform=transform
        self.__cs=out_cs
    def copyFt(self,ft: Feature, row: int):
        geo_new=ft.geometry.affine_transform(self.__transform)
        geo_new.coordsystem=self.__cs
        ft.geometry=geo_new
        return True
def FeatureClone(ft:Feature,new_geo):
    attr_temp=ft.items()
    attr= {}
    for att in attr_temp:
        if att[0]=='+geometry' or att[0]=='+style':
            continue
        attr[att[0]]=att[1]

    return Feature(attr,geometry=new_geo,style=ft.style)
def CreateOutFile(path_out,schema:Schema):
    tab=provider_manager.tab.create_open(path_out,schema)
    return tab
base_point=[]
base_point.append(Pnt(4208062.76,7481837.59))
base_point.append(Pnt(4208248.77,7481842.47))
base_point.append(Pnt(4208233.53,7481702.81))

base_point=[]
base_point.append(Pnt(4208063.13416656,7481837.35311195))
base_point.append(Pnt(4208250.21728733,7481842.46772807))
base_point.append(Pnt(4208234.07705968,7481702.26206446))
reg_points=[]
reg_points.append(Pnt(6513207.56,5591405.35))
reg_points.append(Pnt(6513385.4,5591345.81))
reg_points.append(Pnt(6513322.05,5591220.64))

reg_points=[]
reg_points.append(Pnt(6513208.68,5591404.73))
reg_points.append(Pnt(6513386.23,5591345.55))
reg_points.append(Pnt(6513323.11,5591219.32))
cls_affine=Affine(base_point,reg_points)
transform=cls_affine.getTransform()
cls_ft_tr=FeatureTransform(transform,CoordSystem.from_epsg(3395))
tab_source_path=r"E:\Support\AxiomaTesting\RegisterTab\test_reg_rot_mif_tab.tab"
tab_dest_path=r"E:\Support\AxiomaTesting\RegisterTab\test_reg_rot_mif_tab_aff.tab"
tab_source=provider_manager.openfile(tab_source_path)
shema_source=tab_source.schema
shema_source.coordsystem=CoordSystem.from_epsg(3395)
#tab_out=CreateOutFile(tab_dest_path,shema_source)
destination =provider_manager.tab.get_destination(tab_dest_path, shema_source)
destination.export_from_table(tab_source, copy_schema=False,func_callback=cls_ft_tr.copyFt)
'''
for ft in tab_source.items():
    geo_new=ft.geometry.affine_transform(transform)
    geo_new.coordsystem=shema_source.coordsystem
    ft_new=FeatureClone(ft,geo_new)
    tab_out.insert([ft_new])
'''

