import time
from pathlib import Path

from PySide2.QtCore import QObject, Signal
from axipy import AxipyProgressHandler, Table, Feature, provider_manager, CoordSystem

from RegisterVector.ui.add.add_ui.table_helper import FeatureClone, CacheTable, CreateOutFile, CreateOutTabFileInvert


def isVectorTab(table_source):
    if not isinstance(table_source,Table):
        return False
    name_provider=table_source.provider
    if name_provider!="TabDataProvider":
        return False
    if not table_source.is_spatial:
        return False
    return True
class transformMultiTask(QObject):
    finish_task=Signal()
    update_second_progress=Signal(int)
    def __init__(self,list_source_table:list,transform,cs_out:CoordSystem,bInvertTransform,path_folder_dest):
        super().__init__()
        self.__list_source=list_source_table
        self.__folder_dest=path_folder_dest
        self.__bInvert=bInvertTransform
        self.__transform=transform
        self.__cs_out=cs_out
        self.__count_table=len(list_source_table)
        self.__isCancel=False

    def run(self,ph: AxipyProgressHandler,cls_progessBar):
        curent_transform=self.__transform
        if self.__bInvert:
            ''' Обратное преобразование'''
            curent_transform_е = self.__transform.inverted()
            curent_transform=curent_transform_е[0]
        path_out_folder=Path(self.__folder_dest)
        self.__cls_progress=cls_progessBar
        cls_progessBar.cancel_signal.connect(self.__set_cancel)
        cls_progessBar.setMaxValueMain(self.__count_table)
        self.__ph=ph
        ph.set_max_progress(self.__count_table)
        for j,path_tab in enumerate(self.__list_source):
            if ph.is_canceled():
                break
            self.__cls_progress.setValueMain(j)
            ph.add_progress(1)
            curent_tab=provider_manager.openfile(path_tab)
            if not isVectorTab(curent_tab):
                curent_tab.close()
                continue
            schema_out=curent_tab.schema
            schema_out.coordsystem=self.__cs_out
            name_file_tab=str(Path(path_tab).name)
            path_out_tab=str(path_out_folder.joinpath(name_file_tab))
            if not self.__bInvert:
                tab_out=CreateOutFile(path_out_tab,schema_out)
            else:
                tab_out=CreateOutTabFileInvert(curent_tab,self.__cs_out,curent_transform,path_out_tab)
            cur_cach=CacheTable(tab_out,1000)
            self.update_second_progress.emit(0)
            count_ft_in_tab=curent_tab.count()
            self.__cls_progress.setMaxValue(count_ft_in_tab)
            for i,ft in enumerate(curent_tab.items()):
                if ph.is_canceled():
                    break
                if self.__isCancel:
                    break
                geo_new = ft.geometry.affine_transform(curent_transform)
                geo_new.coordsystem = self.__cs_out
                ft_new = FeatureClone(ft, geo_new)
                cur_cach.add(ft_new)

                self.update_second_progress.emit((i+1))
                # time.sleep(1)
            cur_cach.end()
            tab_out.commit()
            tab_out.close()
            curent_tab.close()
            #time.sleep(1)

        self.finish_task.emit()
        return ph.progress()

    def __set_cancel(self):
        self.__isCancel=True
        self.__ph.cancel()