import json

from PySide2.QtGui import QTransform
from axipy import Pnt, CoordSystem


class Affine:
    def __init__(self,points_base,points_tab):
        self.__points_base=points_base
        self.__points_tab=points_tab
        self.__isValid=False
        self.__calc()
    def __calc(self):
        self._tx=self.__calcCoefficient(self.__points_tab[0],self.__points_tab[1],self.__points_tab[2],self.__points_base[0].x,self.__points_base[1].x,self.__points_base[2].x)
        self._ty = self.__calcCoefficient(self.__points_tab[0], self.__points_tab[1], self.__points_tab[2],
                                    self.__points_base[0].y, self.__points_base[1].y, self.__points_base[2].y)

    def __calcCoefficient(self,pnt1s:Pnt,pnt2s:Pnt,pnt3s:Pnt,v1o,v2o,v3o):
        d1_2x=pnt1s.x-pnt2s.x
        d1_2y=pnt1s.y-pnt2s.y
        d1_3x=pnt1s.x-pnt3s.x
        d1_3y = pnt1s.y - pnt3s.y
        t1_2=v1o-v2o
        t1_3=v1o-v3o
        if d1_3y==0:
            if d1_3x==0:
                self.__isValid=False
                return
            a=t1_3/d1_3x
            if d1_2y==0:
                self.__isValid = False
                return
        else:
            a6=d1_2x-d1_3x*d1_2y/d1_3y
            t6=t1_2-t1_3*d1_2y/d1_3y
            if a6==0:
                self.__isValid=False
                return
            a=t6/a6
        if d1_2y==0:
            b=(t1_3-d1_3x*a)/d1_3y
        else:
            b=(t1_2-d1_2x*a)/d1_2y
        c=v1o-a*pnt1s.x-b*pnt1s.y
        self.__isValid = True

        return a,b,c
    def getTransform(self):
        #qt_trans=QTransform(self._tx[0],self._ty[0],0,self._tx[1],self._ty[1],0,self._tx[2],self._ty[2],0)
        #qt_trans.setMatrix(self._tx[0],self._ty[0],1,self._tx[1],self._ty[1],1,self._tx[2],self._ty[2],1)
        qt_trans=QTransform(self._tx[0],self._ty[0], self._tx[1],self._ty[1], self._tx[2],self._ty[2])
        return qt_trans
    @property
    def isValid(self):
        return self.__isValid
def listPointsToDict(points):
    dict_points=[]
    for i,pt in enumerate(points):
        dict_points.append(point_dict(i+1,pt.x,pt.y))
    return dict_points
def point_dict(id,x,y):
    return {"id":id,"x":x,"y":y}
def descriptTrasformMap(obj_map):
    cs_mi_str=obj_map['cs_mi']
    cs=CoordSystem.from_prj(cs_mi_str)
    points_teg=obj_map['points']
    points=[]
    for tag_pnt in points_teg:
        id=tag_pnt['id']
        x=tag_pnt['x']
        y=tag_pnt['y']
        points.append(Pnt(x,y))
    return cs,points
class DescriptorTransform:
    def __init__(self,cs_base:CoordSystem,points_base_map,cs_points_reg:CoordSystem,points_reg,transform):
        self.__cs_base=cs_base
        self.__points_base=points_base_map
        self.__cs_points_reg=cs_points_reg
        self.__points_reg=points_reg
        self.__qt_transform=transform
    @property
    def transform(self):
        return self.__qt_transform
    @property
    def coordSystemBase(self):
        return self.__cs_base
    @property
    def coordSystemTarget(self):
        return self.__cs_points_reg
    @property
    def pointsBase(self):
        return self.__points_base
    @property
    def pointsRegistration(self):
        return self.__points_reg
    def createJson(self):
        out_description={}
        descriptions_base={}
        descriptions_base['cs_mi']=self.__cs_base.prj
        descriptions_base['cs_wkt'] = self.__cs_base.wkt
        descriptions_base['points']=listPointsToDict(self.__points_base)
        descriptions_reg = {}
        descriptions_reg['cs_mi'] = self.__cs_points_reg.prj
        descriptions_reg['cs_wkt'] = self.__cs_points_reg.wkt
        descriptions_reg['points'] = listPointsToDict(self.__points_reg)
        out_description['base_map']=descriptions_base
        out_description['table_registration'] = descriptions_reg
        json_descriptions=json.dumps(out_description, indent = 4)
        return json_descriptions
    @staticmethod
    def factoryFromJson(path_json):
        data_trasform=None
        with open(path_json, encoding='utf-8') as fh:
            data_trasform = json.load(fh)
        if not "base_map" in data_trasform.keys():
            print("not find key base_map")
            return None
        base_map_info=descriptTrasformMap(data_trasform['base_map'])
        target_table_info=descriptTrasformMap(data_trasform['table_registration'])
        cls_Affine=Affine(base_map_info[1],target_table_info[1])
        csl_description=DescriptorTransform(base_map_info[0],base_map_info[1],target_table_info[0],target_table_info[1],cls_Affine.getTransform())
        return csl_description







