from PySide2.QtCore import QObject, Signal
from axipy import CoordSystem, provider_manager, attr, MapView, Layer, Style, Point, Feature, data_manager, Pnt

from RegisterVector.ui.add.maptool import moveLayerTo
from RegisterVector.ui.add.uiPoints import PointsCntl


class TempRegPoint(QObject):
    update_points_registration=Signal()
    def __init__(self,name,points_cntl:PointsCntl,cs:CoordSystem,style_pnt=None):
        super().__init__()
        self.__name=name
        self.__points=points_cntl
        self.__cs=cs
        self.__initTab()
        self.__ft_points={}
        self.__def_style=style_pnt
        if style_pnt is None:
            self.__def_style=Style.from_mapinfo('Symbol (50, 16711680, 17, "Map Symbols", 1, 0)')
        self.__points.update_pointxy.connect(self.__update_press_XY)
    def __initTab(self):
        definition = {
            'src': '',
            'schema': attr.schema(
                attr.integer('id'),
                coordsystem=self.__cs,

            ),
            'hidden': True,

        }
        self.__table = provider_manager.create(definition)
        self.__table.name = self.__name
    @property
    def table(self):
        return self.__table
    def setPointXY(self,x:float,y:float):
        self.__points.setXY(x,y)
        name_point="point_"+str(self.__points.activeId)
        new_point = Point(x, y, self.__cs)
        new_ft_point = Feature({'id': self.__points.activeId}, geometry=new_point, style=self.__def_style)
        if name_point in self.__ft_points.keys():
            ''' Update point'''
            sql_delete="Delete From "+self.__table.name+" where id="+str(self.__points.activeId)
            data_manager.query_hidden(sql_delete)
            self.__ft_points[name_point] = new_ft_point
            #ft_point= self.__ft_points[name_point]
            #ft_point.geometry=new_point
            #self.__table.update([ft_point])
            #new_ft_point = Feature({'id': self.__points.activeId}, geometry=new_point, style=self.__def_style)
            #self.__ft_points[name_point] = new_ft_point
            #self.__table.insert([new_ft_point])
        else:
            ''' Создам новую точку '''
            #new_ft_point=Feature({'id':self.__points.activeId},geometry=new_point,style=self.__def_style)
            self.__ft_points[name_point]=new_ft_point
        self.__table.insert([new_ft_point])
    def setActiveId(self,id):
        self.__points.setActiveId(id)
    def countPoints(self):
        return len(list(self.__ft_points.keys()))
    def Points(self):
        list_points=[]
        if len(list(self.__ft_points.keys()))<3:
            return None
        for id in range(len(list(self.__ft_points.keys()))):
            name_point="point_"+str(id+1)
            ft=self.__ft_points[name_point]
            list_points.append(Pnt(ft.geometry.x,ft.geometry.y))
        return list_points
    def closeTable(self):
        self.disconnectPoints()
        self.__points.clearAll()
        self.__ft_points = {}
        #sql_delete_all="Delete From "+self.__table.name
        #data_manager.query_hidden(sql_delete_all)
        #count=self.__table.count()
        self.__table.rollback()
        self.__table.close()
    def __update_press_XY(self,id,x,y):
        self.__points.setActiveId(id)
        self.setPointXY(x,y)
        self.update_points_registration.emit()
        print("Tab Points ID=" + str(id) + " X=" + str(x) + " Y=" + str(y))
    def disconnectPoints(self):
        self.__points.update_pointxy.disconnect(self.__update_press_XY)
def createTableAndAddToMap(map_view:MapView,cls_points):
    name="tmp_point_"+map_view.title
    temp_tab_cls=TempRegPoint(name,cls_points,map_view.coordsystem)
    layer_temp=Layer.create(temp_tab_cls.table)
    layer_temp.label.text = 'id'
    layer_temp.label.visible = True
    map_view.map.layers.append(layer_temp)
    moveLayerTo(map_view.map,layer_temp.title,0)
    return temp_tab_cls
