import math

from PySide2.QtCore import QObject, Signal
from PySide2.QtGui import QDoubleValidator
from PySide2.QtWidgets import QWidget, QLineEdit, QRadioButton, QButtonGroup


def getNumPointEndXY(name_obj):
    att_name=name_obj.split('_')
    if len(att_name)<4:
        return None,None
    return att_name[2],att_name[3]
def getNumPointButton(name_obj):
    att_name=name_obj.split('_')
    if len(att_name)<3:
        return None,None
    return att_name[1],att_name[2]
class CntlPoint(QObject):
    update_value_XY=Signal(int,float,float)
    def __init__(self,id,ln_editX,ln_editY):
        super().__init__()
        self.__id=id
        self.__lnx=ln_editX
        self.__lny=ln_editY
        self.__lnx.setValidator(QDoubleValidator())
        self.__lny.setValidator(QDoubleValidator())
        self.__lnx.returnPressed.connect(self.__end_edit_x)
        self.__lny.returnPressed.connect(self.__end_edit_x)
    def updateXY(self,x:float,y:float):
        self.__lnx.setText(str(x))
        self.__lnx.setCursorPosition(0)
        self.__lny.setText(str(y))
        self.__lny.setCursorPosition(0)

    def __end_edit_x(self):
        value_x=float(self.__lnx.text())
        value_y = float(self.__lny.text())
        self.update_value_XY.emit(self.__id,value_x,value_y)
        print("Press Enter X Point ID="+str(self.__id))
    def __end_edit_y(self):
        value_x = float(self.__lnx.text())
        value_y = float(self.__lny.text())
        self.update_value_XY.emit(self.__id, value_x, value_y)
        print("Press Enter Y Point ID="+str(self.__id))
class PointsCntl(QObject):
    update_pointxy=Signal(int,float,float)
    def __init__(self):
        super().__init__()
        self.__pointCntl=[]
        self.__id_active=1

    def add(self,point:CntlPoint):
        point.update_value_XY.connect(self.__change_value_xy)
        self.__pointCntl.append(point)

    def __change_value_xy(self,id,x,y):
        ''' Обработка изменений с клавиатуры координат'''
        print("ID="+str(id)+" X="+str(x)+" Y="+str(y))
        self.update_pointxy.emit(id,x,y)
    def setXY(self,x,y):
        #self.update_pointxy.emit(self.__id_active,x,y)
        self.__pointCntl[self.__id_active-1].updateXY(x,y)
    @property
    def activeId(self):
        return self.__id_active
    def setActiveId(self,id):
        self.__id_active=id
    def clearAll(self):
        for pnt in self.__pointCntl:
            pnt.updateXY(0,0)
def factoryClsPoinsValue(parent_widget:QWidget):
    all_lineedits=parent_widget.findChildren(QLineEdit)
    temp_pnt_cls={'1':{},'2':{},'3':{}}
    for w_line_edit in all_lineedits:
        object_name=w_line_edit.objectName()
        name_axis,id=getNumPointEndXY(object_name)
        temp_pnt_cls[id][name_axis]=w_line_edit
    cls_Points=PointsCntl()
    for i in range(1,4):
        sid=str(i)
        pnts_widget=temp_pnt_cls[sid]
        cls_Points.add(CntlPoint(i,pnts_widget['x'],pnts_widget['y']))
    return cls_Points
def getButtonGroup(parent_widget:QWidget):
    all_QRadioButton = parent_widget.findChildren(QRadioButton)
    btn_grp = QButtonGroup()
    btn_grp.setExclusive(True)
    for button in all_QRadioButton:
        object_name = button.objectName()
        name_axis, id = getNumPointButton(object_name)
        if id is None:
            continue
        btn_grp.addButton(button, id=int(id))
    return btn_grp
def CollinearPoints(list_points)->bool:
    ''' Проверяем Коллинеарные точки ( на горизонтали или вертикали)'''
    pt1=list_points[0]
    pt2 = list_points[1]
    pt3 = list_points[2]
    dx1=pt1.x-pt2.x
    dx2=pt2.x-pt3.x
    if math.fabs(dx1-dx2)<0.0000000000000001:
        return 'v',True
    dy1=pt1.y-pt2.y
    dy2=pt2.y-pt3.y
    if math.fabs(dy1-dy2)<0.0000000000000001:
        return 'g',True
    return None,False





