import io
from pathlib import Path

from PySide2 import QtCore
from PySide2.QtWidgets import QDialog, QFileDialog
from axipy import data_manager

from RegisterVector.ui.add.add_ui.runTransform import runTransformTab
from RegisterVector.ui.add.affine_transform import Affine, DescriptorTransform
from RegisterVector.ui.helperUI import load_ui
from RegisterVector.ui.sourceControl import SourceMap, SourceTable
from RegisterVector.ui.widgetMapCntrl import MapPointCntrl


class RegisterVector(QDialog):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.ui=load_ui("widgetMapCntrlTwoMap1.ui",parent)
        self.ui.setWindowFlag(QtCore.Qt.WindowContextHelpButtonHint, False)
        self.__cls_descriptions=None
        self.__cls_map_source=SourceMap()
        self.__cls_table_source = SourceTable()

        self.__cntl_map=MapPointCntrl({'map':'Базовая карта','source':'Карта'},self.__cls_map_source,self.ui.group_base_map)
        self.__cntl_map_table = MapPointCntrl({'map':'Карта регистрируемой таблицы','source':'Таблица'},self.__cls_table_source, self.ui.group_table_map)
        self.ui.layoutMap.addWidget(self.__cntl_map.ui)
        self.ui.layoutMap_2.addWidget(self.__cntl_map_table.ui)
        self.__cntl_map.count_point_ready.connect(self.__change_points_reg_base_map)
        self.__cntl_map_table.count_point_ready.connect(self.__change_points_reg_tab_map)
        self.ui.pb_save.clicked.connect(self.__save_transform)
        self.ui.pb_run.clicked.connect(self.__run_transform)
        self.ui.pb_load.clicked.connect(self.__load_transform)
        self.ui.pb_close.clicked.connect(self.__close)
        self.ui.finished.connect(self.__close)
        self.__count_point_in_map=0
        self.__count_point_in_tab = 0
    def __change_points_reg_base_map(self,count_point):
        self.__count_point_in_map=count_point
        self.__enableRun()
    def __change_points_reg_tab_map(self,count_point):
        self.__count_point_in_tab=count_point
        self.__enableRun()
    def __enableRun(self):
        isEnabled=False
        if self.__count_point_in_map==3 and self.__count_point_in_tab==3:
            self.ui.pb_run.setEnabled(True)
            isEnabled=True
        else:
            self.ui.pb_run.setEnabled(False)
            self.ui.pb_save.setEnabled(False)
        if isEnabled:
            points_base=self.__cntl_map.getPointsRegistration()
            points_tab=self.__cntl_map_table.getPointsRegistration()
            cls_affine=Affine(points_base,points_tab)
            if not cls_affine.isValid:
                self.ui.pb_run.setEnabled(False)
                return
            qt_transform=cls_affine.getTransform()
            cs_base_map=self.__cntl_map.getCoordSysMap()
            cs_table_map=self.__cntl_map_table.getCoordSysMap()
            self.__cls_descriptions=DescriptorTransform(cs_base_map,points_base,cs_table_map,points_tab,qt_transform)
            self.ui.pb_save.setEnabled(True)

    def __save_transform(self):
        curent_name_tab_select=self.__cntl_map_table.getSelectSource()
        tab=data_manager.find(curent_name_tab_select)
        tab_select=tab.properties['fileName']
        path_tab=Path(tab_select)
        out_path_file=str(path_tab.joinpath(tab.name+".js_transform"))
        ext_file = '*.js_transform'
        dlgSaveAs=QFileDialog()
        dlgSaveAs.setDirectory(str(path_tab.parent))
        name_save_file = dlgSaveAs.getSaveFileName(self.ui, 'Сохранить параметры регистрации', out_path_file, ext_file)
        if name_save_file is not None and (name_save_file[0]!=""):
            json_out = self.__cls_descriptions.createJson()
            with io.open(name_save_file[0], 'w', encoding='utf8') as f:
                f.write(json_out)
    def __run_transform(self):
        curent_name_tab_select = self.__cntl_map_table.getSelectSource()
        tab = data_manager.find(curent_name_tab_select)
        tab_select = tab.properties['fileName']
        path_tab = Path(tab_select)
        out_path_file = str(path_tab.joinpath(tab.name + "_tr.tab"))
        ext_file = '*.tab'
        dlgSaveAs = QFileDialog()
        dlgSaveAs.setDirectory(str(path_tab.parent))
        name_save_file = dlgSaveAs.getSaveFileName(self.ui, 'Сохранить таблицу', out_path_file, ext_file)
        if name_save_file is not None and (name_save_file[0]!=""):
            path_out_tab=name_save_file[0]
            runTransformTab(tab,self.__cls_descriptions,path_out_tab,self.ui)

    def __load_transform(self):
        curent_name_tab_select = self.__cntl_map_table.getSelectSource()
        tab = data_manager.find(curent_name_tab_select)
        tab_select = tab.properties['fileName']
        path_tab = Path(tab_select)
        out_path_file = str(path_tab.joinpath(tab.name + ".js_transform"))
        ext_file = '*.js_transform'
        dlgSaveAs = QFileDialog()
        dlgSaveAs.setDirectory(str(path_tab.parent))
        path_load=dlgSaveAs.getOpenFileName(self.ui, 'Сохранить таблицу', out_path_file, ext_file)
        if path_load is not None and (path_load[0]!=""):
            path_load_js_tr_file=path_load[0]
            self.__cls_descriptions=DescriptorTransform.factoryFromJson(path_load_js_tr_file)
            self.__cntl_map.setPointsRegistration(self.__cls_descriptions.pointsBase)
            self.__cntl_map_table.setPointsRegistration(self.__cls_descriptions.pointsRegistration)
    def show(self):
        self.ui.show()
    def exec_(self):
        self.ui.exec_()
    def __close(self):
        self.__cntl_map.deleteMapWidget()
        self.__cntl_map_table.deleteMapWidget()
        self.ui.close()
