import os
from pathlib import Path

from PySide2 import QtCore
from PySide2.QtGui import QIcon
from PySide2.QtWidgets import QDialog, QFileDialog

from RegisterVector.ui.add.add_ui.dlg_progressMultiTask import dlgProgressBarMultiTask
from RegisterVector.ui.add.add_ui.taskTransformMultiTable import transformMultiTask
from RegisterVector.ui.add.affine_transform import DescriptorTransform
from RegisterVector.ui.helperUI import load_ui



class DlgPacketTransfromTab(QDialog):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.ui=load_ui("dlg_multi_transform2.ui",parent)
        self.__list_source_tab=None
        self.__file_transform=None
        self.__dest_folder=None
        curent_path = str(Path(__file__).parent)

        self.__path_icon = os.path.join(curent_path, "icons")
        # path_add=os.path.join(self.__path_icon, "add.png")
        self.ui.pb_select_tabs.setIcon(QIcon(os.path.join(self.__path_icon, "open-32.png")))
        self.ui.pb_select_transform.setIcon(QIcon(os.path.join(self.__path_icon, "open-32.png")))
        self.ui.pb_saveas.setIcon(QIcon(os.path.join(self.__path_icon, "saveas.png")))
        self.ui.pb_select_tabs.clicked.connect(self.__selectFiles)
        self.ui.pb_select_transform.clicked.connect(self.__loadTransform)
        self.ui.pb_saveas.clicked.connect(self.__select_out_folder)
        self.ui.pb_close.clicked.connect(self.__close)
        self.ui.finished.connect(self.__close)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.setWindowFlag(QtCore.Qt.WindowContextHelpButtonHint, False)
        self.__dlgFile = QFileDialog()
        self.__dlgFile.setDirectory(str(Path.home()))
    def show(self):
        self.ui.show()
    def exec_(self):
        self.ui.exec_()
    def __selectFiles(self):
        filter_mask = "MapInfo Tab (*.tab)"
        files_tab=self.__dlgFile.getOpenFileNames(self.ui,"Выбор таблиц для преобразования",filter=filter_mask)
        if files_tab is not None and (len(files_tab[0])>0):
            self.ui.listWidget_source.clear()
            self.__list_source_tab=files_tab[0]
            path_folder=Path(self.__list_source_tab[0]).parent
            self.__dlgFile.setDirectory(str(path_folder))
            for tab in self.__list_source_tab:
                name_tab=str(Path(tab).name)
                self.ui.listWidget_source.addItem(name_tab)
            self.__isEnabledRun()

    def __loadTransform(self):
        filter_mask = "Параметры преобразования (*.js_transform)"
        files_tab = self.__dlgFile.getOpenFileName(self.ui, "Выбор файла параметров преобразования", filter=filter_mask)
        if files_tab is not None and (len(files_tab[0]) > 0):
            self.__file_transform=files_tab[0]
            self.ui.ln_transform_file.setText(self.__file_transform)
            self.ui.ln_transform_file.setCursorPosition(0)
            self.__isEnabledRun()
    def __select_out_folder(self):

        folder=self.__dlgFile.getExistingDirectory(self.ui,"Директория для результирующих таблицы")
        if len(folder)>0:
            self.ui.ln_dest_folder.setText(folder)
            self.ui.ln_dest_folder.setCursorPosition(0)
            self.__dest_folder=folder
            self.__isEnabledRun()
    def __isEnabledRun(self):
        if self.__dest_folder is not None and (self.__file_transform is not None) and (self.__list_source_tab is not None):
            self.ui.pb_run.setEnabled(True)
        else:
            self.ui.pb_run.setEnabled(False)
    def __run(self):
        cls_descripor_transform=DescriptorTransform.factoryFromJson(self.__file_transform)
        transform=cls_descripor_transform.transform
        cs_out=cls_descripor_transform.coordSystemBase
        bInvertTransform=self.ui.ch_invert_transform.isChecked()
        if not bInvertTransform:
            cls_multiTask = transformMultiTask(self.__list_source_tab, transform,cs_out,False, self.__dest_folder)
        else:
            cls_multiTask = transformMultiTask(self.__list_source_tab, transform, cls_descripor_transform.coordSystemTarget, True, self.__dest_folder)
        lcs_dlg = dlgProgressBarMultiTask(cls_multiTask, self.ui)
        self.__enableRunClose(False)
        lcs_dlg.show()
        self.__enableRunClose(True)
        jkl=0
    def __enableRunClose(self,value):
        self.ui.pb_run.setEnabled(value)
        self.ui.pb_close.setEnabled(value)
        #self.ui.setWindowFlag(QtCore.Qt.WindowCloseButtonHint,value)

    def __close(self):
        self.ui.close()





