from enum import Enum

from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget, QVBoxLayout

from RegisterVector.ui.add.tempTablePoint import createTableAndAddToMap
from RegisterVector.ui.add.uiPoints import getButtonGroup, factoryClsPoinsValue, CollinearPoints
from RegisterVector.ui.helperUI import load_ui
from RegisterVector.ui.mapWidget import dlgMapWidget


class TypeControlMap(Enum):
    Map=1
    Table=2
class MapPointCntrl(QWidget):

    count_point_ready=Signal(int)

    def __init__(self,name_items,cls_map, parent=None):
        super().__init__(parent)
        self.__name_items=name_items
        self.__curent_map_view =None
        self.__tempPointsTabBaseMap=None
        self.__cls_pointsCntrl_base_map = None
        self.__cls_map=cls_map
        self.ui = load_ui("widgetMapCntrl.ui", parent)
        self.ui.label_source.setText(self.__name_items['source'])
        self.__layoutMap=QVBoxLayout()
        self.__layoutMap.setContentsMargins(0, 0, 0, 0)
        self.__cntl_map_view=None
        self.ui.cmb_base_maps.currentIndexChanged.connect(self.__change_map)
        self.__button_grp_base_map = getButtonGroup(self.ui.group_base_map_points)
        self.__button_grp_base_map.buttonClicked.connect(self.__select_edit_point_base)
        self.__initListSource()
    def __initListSource(self):
        for item_source in self.__cls_map.items():
            self.ui.cmb_base_maps.addItem(item_source)
        self.ui.cmb_base_maps.setCurrentIndex(0)

    def __change_map(self,index):
        name_map=self.ui.cmb_base_maps.itemText(index)
        self.__loadNewMap(name_map)
    def __loadNewMap(self,name):
        self.count_point_ready.emit(0)
        if self.__cls_pointsCntrl_base_map is None:
            self.__cls_pointsCntrl_base_map = factoryClsPoinsValue(self.ui.group_base_map_points)

        new_map_view=self.__cls_map.getMapView(name)
        if self.__tempPointsTabBaseMap is not None:
            self.__tempPointsTabBaseMap.update_points_registration.disconnect(self.__enableReady)
            self.__tempPointsTabBaseMap.closeTable()
        self.__tempPointsTabBaseMap = createTableAndAddToMap(new_map_view, self.__cls_pointsCntrl_base_map)
        self.__tempPointsTabBaseMap.update_points_registration.connect(self.__enableReady)
        if self.__curent_map_view is None:
            self.__curent_map_view = new_map_view
            self.__cntl_map_view=dlgMapWidget(self.__curent_map_view,self.ui.frame_map)
            self.__layoutMap.addWidget(self.__cntl_map_view)
            self.ui.frame_map.setLayout(self.__layoutMap)
            self.__cntl_map_view.click_map.connect(self.__get_coord_base)
        else:
            #self.__curent_map_view.mouse_moved.disconnect(self.__mouse_move_in_map)
            #self.__clearLayout(self.__layoutMap)
            '''
            try:
                self.__curent_map_view.close()
            except:
                pass
            '''
            self.__curent_map_view = new_map_view
            self.__cntl_map_view.setNewMap(self.__curent_map_view)
            #self.__layoutMap.addWidget(self.__cntl_map_view)
            self.ui.frame_map.setLayout(self.__layoutMap)
            #self.__cntl_map_view.click_map.connect(self.__get_coord_base)

        #self.__curent_map_view.mouse_moved.connect(self.__mouse_move_in_map)
    def show(self):
        self.ui.show()

    def __select_edit_point_base(self, button):
        print(self.__button_grp_base_map.id(button), button.text())
        self.__tempPointsTabBaseMap.setActiveId(self.__button_grp_base_map.id(button))
    def __get_coord_base(self,x,y):
        patern_xy = "x: {x:.5f} y: {y:.5f}"

        #print(patern_xy.format(x=x, y=y))
        self.__tempPointsTabBaseMap.setPointXY(x,y)
        self.__enableReady()
    def setPointsRegistration(self,points):
        for i,pnt in enumerate(points):
            self.__tempPointsTabBaseMap.setActiveId(i+1)
            self.__tempPointsTabBaseMap.setPointXY(pnt.x,pnt.y)
        self.__enableReady()
    def __enableReady(self):
        count_point_base = self.__tempPointsTabBaseMap.countPoints()
        if count_point_base==3:
            points=self.__tempPointsTabBaseMap.Points()
            direction,isCollinear=CollinearPoints(points)
            if not isCollinear:
                self.count_point_ready.emit(3)
            else:
                pass
    def __clearLayout(self,layout):
        if layout is not None:
            while layout.count():
                child = layout.takeAt(0)
                if child.widget() is not None:
                    child.widget().deleteLater()
                elif child.layout() is not None:
                    self.__clearLayout(child.layout())
    def deleteMapWidget(self):
        self.__clearLayout(self.__layoutMap)
        if self.__curent_map_view is None:
            self.__curent_map_view.close()
    def getPointsRegistration(self):
        return self.__tempPointsTabBaseMap.Points()
    def getCoordSysMap(self):
        return self.__curent_map_view.coordsystem
    def testSetMapView(self,map_view):
        self.__curent_map_view=map_view
        self.__layoutMap.addWidget(self.__curent_map_view.widget)
        self.ui.frame_map.setLayout(self.__layoutMap)
    def getSelectSource(self):
        return self.ui.cmb_base_maps.currentText()

